package edu.uic.cs474.f21.a4;

enum Operator { ADD, SUB, MUL, DIV };
public abstract class Expression {
    static class BinOp extends Expression{
        final Operator op;
        final Expression leftOperand, rightOperand;

        BinOp(Operator op, Expression left, Expression right){
            this.op = op;
            this.leftOperand = left;
            this.rightOperand = right;
        }
    }

    static class IntConst extends Expression{
        final int intValue;

        IntConst(int i){
            this.intValue = i;
        }
    }

    static class VarExp extends Expression{
        final Name name;  // x + 5

        VarExp(Name name){
            this.name = name;
        }
    }

    /* let x = 5 in x + 3 */
    /* { x = 5; x + 3 } */
    static class LetExp extends Expression{
        Name varName;
        Expression initializer;
        Expression body;

        LetExp(Name varName, Expression initializer, Expression body){
            this.varName = varName;
            this.initializer = initializer;
            this.body = body;
        }
    }

    static class FunDefExp extends Expression{
        final Name funName;
        final Name[] parameters;
        final Expression funBody;
        final Expression body;

        FunDefExp(Name funName, Name[] parameters, Expression funBody, Expression body) {
            this.funName = funName;
            this.parameters = parameters;
            this.funBody = funBody;
            this.body = body;
        }
    }

    static class FunCallExp extends Expression{
        final Name funName;
        final Expression[] arguments;

        FunCallExp(Name funName, Expression[] arguments) {
            this.funName = funName;
            this.arguments = arguments;
        }
    }
}