package edu.uic.cs474.f21.a4;

import java.util.*;
import java.util.function.Function;

public class Interpreter {
    // Binding has a variable name and a value

    // x = 4; y = 5; z = 3;

/*    {
        int x
        {
            int y
            {
                int z
            }
        }
    }*/

    /* mutable environment interface:
       We can push a new binding (set x to 5 temporarily)
       We can look up the current value of a variable
       We can pop off a binding once it goes out of scope
     */

    /* immutable environment interface:
       We can allocate a new environment that has one more binding than the current one
       We can look up the current value of a variable
     */
    static Value eval(Expression e, Environment environment){  /* we take a string, and see if it's 3 + 4 + 3, ... */
        Value result = null;
        if(e instanceof Expression.IntConst){
            Expression.IntConst constExpr = (Expression.IntConst) e;
            result = new Value.IntValue(constExpr.intValue);
        }
        else if(e instanceof Expression.VarExp){
            Expression.VarExp varExp = (Expression.VarExp) e;
            result = environment.lookup(varExp.name);
        }
        else if(e instanceof Expression.LetExp){
            Expression.LetExp letExp = (Expression.LetExp) e;

            Environment newEnv = environment.bind(letExp.varName, eval(letExp.initializer, environment));
            result = eval(letExp.body, newEnv);
//            environment = /* whatever environment was before */;
        }
        else if(e instanceof Expression.FunCallExp){
            Expression.FunCallExp callExp = (Expression.FunCallExp) e;

            Value.FunDefValue toBeCalled = (Value.FunDefValue) environment.lookup(callExp.funName);

            Environment newEnv = environment;

            // params: x, y            arguments: 3, 4 + 5
            for(int i = 0; i < callExp.arguments.length; i++){
                newEnv = newEnv.bind(toBeCalled.parameters[i], eval(callExp.arguments[i], environment));
            }

            result = eval(toBeCalled.funBody, newEnv);
        }
        else if(e instanceof Expression.FunDefExp){
            Expression.FunDefExp funDefExp = (Expression.FunDefExp) e;

            Value.FunDefValue newFun = new Value.FunDefValue(funDefExp.funName, funDefExp.parameters, funDefExp.funBody);

            Environment newEnv = environment.bind(funDefExp.funName, newFun);
            result = eval(funDefExp.body, newEnv);
//            environment = /* whatever environment was before */;
        }
        else if(e instanceof Expression.BinOp){
            Expression.BinOp binExpr = (Expression.BinOp) e;

            Value.IntValue leftValue = (Value.IntValue) eval(binExpr.leftOperand, environment);
            Value.IntValue rightValue = (Value.IntValue) eval(binExpr.rightOperand, environment);

            switch(binExpr.op){
                case ADD: result = new Value.IntValue(leftValue.val + rightValue.val); break;
                case SUB: result = new Value.IntValue(leftValue.val - rightValue.val); break;
                case MUL: result = new Value.IntValue(leftValue.val * rightValue.val); break;
                case DIV: result = new Value.IntValue(leftValue.val / rightValue.val); break;
            }

        }
        // else ...
        else {
            throw new Error("unknown expression type" + e);
        }

        return result;
    }
}